@@ -106,6 +106,7 @@ pub struct DecodeReaderBytesBuilder {
     utf8_passthru: bool,
     bom_override: bool,
     strip_bom: bool,
+    bom_sniffing: bool,
 }
 
 impl Default for DecodeReaderBytesBuilder {
@@ -127,6 +128,7 @@ impl DecodeReaderBytesBuilder {
             utf8_passthru: false,
             bom_override: false,
             strip_bom: false,
+            bom_sniffing: true,
         }
     }
 
@@ -161,12 +163,14 @@ impl DecodeReaderBytesBuilder {
         }
         let encoding = self.encoding
             .map(|enc| enc.new_decoder_with_bom_removal());
-        // No need to do BOM detection if we have an explicit encoding.
-        let has_detected = !self.bom_override && encoding.is_some();
+
+        // No need to do BOM detection if we opt out of it or have an explicit
+        // encoding.
+        let has_detected =
+            !self.bom_sniffing || (!self.bom_override && encoding.is_some());
+
         let peeker =
-            if self.utf8_passthru && self.strip_bom {
-                // We only need to do this when utf8_passthru is enabled
-                // because we otherwise setup encoding_rs to strip the BOM.
+            if self.strip_bom {
                 BomPeeker::without_bom(rdr)
             } else {
                 BomPeeker::with_bom(rdr)
@@ -255,7 +259,8 @@ impl DecodeReaderBytesBuilder {
     /// Whether or not to always strip a BOM if one is found.
     ///
     /// When this is enabled, if a BOM is found at the beginning of a stream,
-    /// then it is ignored. This applies even when `utf8_passthru` is enabled.
+    /// then it is ignored. This applies even when `utf8_passthru` is enabled
+    /// or if `bom_sniffing` is disabled.
     ///
     /// This is disabled by default.
     ///
@@ -312,6 +317,24 @@ impl DecodeReaderBytesBuilder {
         self.bom_override = yes;
         self
     }
+
+    /// Enable BOM sniffing
+    ///
+    /// When this is enabled and an explicit encoding is not set, the decoder
+    /// will try to detect the encoding with BOM.
+    ///
+    /// When this is disabled and an explicit encoding is not set, the decoder
+    /// will treat the input as raw bytes. The bytes will be passed through
+    /// unchanged, including any BOM that may be present.
+    ///
+    /// This is enabled by default.
+    pub fn bom_sniffing(
+        &mut self,
+        yes: bool,
+    ) -> &mut DecodeReaderBytesBuilder {
+        self.bom_sniffing = yes;
+        self
+    }
 }
 
 /// An implementation of `io::Read` that transcodes to UTF-8 in a streaming
@@ -739,6 +762,62 @@ mod tests {
         assert_eq!(got, b"abcdefgh");
     }
 
+    #[test]
+    fn trans_utf16_no_sniffing() {
+        let srcbuf = vec![
+            0xFF, 0xFE,
+            0x61, 0x00,
+        ];
+        let rdr = DecodeReaderBytesBuilder::new()
+            .bom_sniffing(false)
+            .build(&*srcbuf);
+        let got: Vec<u8> = rdr.bytes().map(|res| res.unwrap()).collect();
+        assert_eq!(got, srcbuf);
+    }
+
+    #[test]
+    fn trans_utf16_no_sniffing_strip_bom() {
+        let srcbuf = vec![
+            0xFF, 0xFE,
+            0x61, 0x00,
+        ];
+        let rdr = DecodeReaderBytesBuilder::new()
+            .bom_sniffing(false)
+            .strip_bom(true)
+            .build(&*srcbuf);
+        let got: Vec<u8> = rdr.bytes().map(|res| res.unwrap()).collect();
+        assert_eq!(got, &[0x61, 0x00]);
+    }
+
+    #[test]
+    fn trans_utf16_no_sniffing_encoding_override() {
+        let srcbuf = vec![
+            0xFF, 0xFE,
+            0x61, 0x00,
+        ];
+        let rdr = DecodeReaderBytesBuilder::new()
+            .bom_sniffing(false)
+            .encoding(Some(encoding_rs::UTF_16LE))
+            .build(&*srcbuf);
+        let got: Vec<u8> = rdr.bytes().map(|res| res.unwrap()).collect();
+        assert_eq!(got, b"a");
+    }
+
+    #[test]
+    fn trans_utf16_no_sniffing_encoding_override_strip_bom() {
+        let srcbuf = vec![
+            0xFF, 0xFE,
+            0x61, 0x00,
+        ];
+        let rdr = DecodeReaderBytesBuilder::new()
+            .bom_sniffing(false)
+            .strip_bom(true)
+            .encoding(Some(encoding_rs::UTF_16LE))
+            .build(&*srcbuf);
+        let got: Vec<u8> = rdr.bytes().map(|res| res.unwrap()).collect();
+        assert_eq!(got, b"a");
+    }
+
     // Test transcoding with a minimal buffer using byte oriented APIs.
     #[test]
     fn trans_utf16_minimal_buffer_byte_api() {
